// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.
#include "MirrorPluginToolbar.h"
#include "MirrorPluginToolbarStyle.h"
#include "MirrorPluginToolbarCommands.h"
#include "Misc/MessageDialog.h"
#include "Framework/MultiBox/MultiBoxBuilder.h"

#include "Editor.h"
#include "LevelEditor.h"

static const FName MirrorPluginToolbarTabName("MirrorPluginToolbar");

#define LOCTEXT_NAMESPACE "FMirrorPluginToolbarModule"

void FMirrorPluginToolbarModule::StartupModule()
{
	// This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module
	
	FMirrorPluginToolbarStyle::Initialize();
	FMirrorPluginToolbarStyle::ReloadTextures();

	FMirrorPluginToolbarCommands::Register();
	
	PluginCommands = MakeShareable(new FUICommandList);

	PluginCommands->MapAction(
		FMirrorPluginToolbarCommands::Get().PluginAction,
		FExecuteAction::CreateRaw(this, &FMirrorPluginToolbarModule::PluginButtonClicked),
		FCanExecuteAction());
		
	FLevelEditorModule& LevelEditorModule = FModuleManager::LoadModuleChecked<FLevelEditorModule>("LevelEditor");
	
	{
		TSharedPtr<FExtender> MenuExtender = MakeShareable(new FExtender());
		MenuExtender->AddMenuExtension("WindowLayout", EExtensionHook::After, PluginCommands, FMenuExtensionDelegate::CreateRaw(this, &FMirrorPluginToolbarModule::AddMenuExtension));

		LevelEditorModule.GetMenuExtensibilityManager()->AddExtender(MenuExtender);
	}
	
	{
		TSharedPtr<FExtender> ToolbarExtender = MakeShareable(new FExtender);
		ToolbarExtender->AddToolBarExtension("Settings", EExtensionHook::After, PluginCommands, FToolBarExtensionDelegate::CreateRaw(this, &FMirrorPluginToolbarModule::AddToolbarExtension));
		
		LevelEditorModule.GetToolBarExtensibilityManager()->AddExtender(ToolbarExtender);
	}
}

void FMirrorPluginToolbarModule::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
	FMirrorPluginToolbarStyle::Shutdown();

	FMirrorPluginToolbarCommands::Unregister();
}

void FMirrorPluginToolbarModule::PluginButtonClicked()
{
	// Put your "OnButtonClicked" stuff here

	TArray<FAssetData> Selections;
	GEditor->GetContentBrowserSelections(Selections);
	int SelectionCount = 0;
	USkeletalMesh* SkeletalMesh = nullptr;
	UPackage* SkeletalMeshPackage = nullptr;
	for (int i=0; i<Selections.Num(); i++) {
		SelectionCount++;
		FName AssetClass = Selections[i].AssetClass;
		if (AssetClass.Compare(FName(TEXT("SkeletalMesh"))) == 0.0) {
			SkeletalMesh = (USkeletalMesh*) Selections[i].GetAsset();
			SkeletalMeshPackage = Selections[i].GetPackage();
		}
	}

	if (SelectionCount == 0) {
		FText DialogText = FText::FromString(
			TEXT("Nothing is selected. Please select a Skeletal Mesh in the Content Browser.")
		);
		FMessageDialog::Open(EAppMsgType::Ok, DialogText);
	}
	else if (SelectionCount > 1) {
		FText DialogText = FText::FromString(
			TEXT("You have selected too many objects. Please select a single Skeletal Mesh in the Content Browser.")
		);
		FMessageDialog::Open(EAppMsgType::Ok, DialogText);
	}
	else if (SkeletalMesh == nullptr) {
		FText DialogText = FText::FromString(
			TEXT("You have selected the wrong object. Please select a Skeletal Mesh in the Content Browser.")
		);
		FMessageDialog::Open(EAppMsgType::Ok, DialogText);
	}
	else {
		SkeletalMesh->InitBoneMirrorInfo();

		FReferenceSkeleton RefSkeleton = SkeletalMesh->RefSkeleton;

		TArray<FBoneMirrorExport> MirrorTable;
		SkeletalMesh->ExportMirrorTable(MirrorTable);
		int SkeletonLoopCount = MirrorTable.Num();

		for (int32 i=0; i<SkeletonLoopCount; i++) {
			FBoneMirrorExport* MirrorExport = &MirrorTable[i];
			FName RefBoneName = MirrorExport->BoneName;
			FString RefBoneNameStr = RefBoneName.ToString();

			// Determine if bone name has a mirrored partner and what the partner reversed direction suffix/prefix is
			int NameTrimOffset = 0;
			FString ReversedDirection = FString(TEXT(""));
			for (int32 i=0; i<10; i++) {
				FString StartString = RefBoneNameStr.Left(2);
				FString EndString = RefBoneNameStr.Right(2);
				FString NumericCheck = FString(TEXT(""));
				if (i > 0) {
					StartString = RefBoneNameStr.Left(3);
					EndString = RefBoneNameStr.Right(3);
					NumericCheck = FString::FromInt(i);
				}
				if (i == 0) {
					if (StartString.Compare(FString(TEXT("l_"))) == 0.0) {
						NameTrimOffset = 2;
						ReversedDirection = FString(TEXT("r_"));
						break;
					}
					else if (StartString.Compare(FString(TEXT("r_"))) == 0.0) {
						NameTrimOffset = 2;
						ReversedDirection = FString(TEXT("l_"));
						break;
					}
					else if (StartString.Compare(FString(TEXT("L_"))) == 0.0) {
						NameTrimOffset = 2;
						ReversedDirection = FString(TEXT("R_"));
						break;
					}
					else if (StartString.Compare(FString(TEXT("R_"))) == 0.0) {
						NameTrimOffset = 2;
						ReversedDirection = FString(TEXT("L_"));
						break;
					}
				}
				if (EndString.Compare(FString(TEXT("_l")) + NumericCheck) == 0.0) {
					NameTrimOffset = (i > 0) ? -3 : -2;
					ReversedDirection = FString(TEXT("_r")) + NumericCheck;
					break;
				}
				else if (EndString.Compare(FString(TEXT("_r")) + NumericCheck) == 0.0) {
					NameTrimOffset = (i > 0) ? -3 : -2;
					ReversedDirection = FString(TEXT("_l")) + NumericCheck;
					break;
				}
				else if (EndString.Compare(FString(TEXT("_L")) + NumericCheck) == 0.0) {
					NameTrimOffset = (i > 0) ? -3 : -2;
					ReversedDirection = FString(TEXT("_R")) + NumericCheck;
					break;
				}
				else if (EndString.Compare(FString(TEXT("_R")) + NumericCheck) == 0.0) {
					NameTrimOffset = (i > 0) ? -3 : -2;
					ReversedDirection = FString(TEXT("_L")) + NumericCheck;
					break;
				}
			}

			/*
			bool IsRefLeftSuffix = RefBoneNameStr.Right(2).Compare(FString(TEXT("_l"))) == 0.0;
			bool IsRefRightSuffix = RefBoneNameStr.Right(2).Compare(FString(TEXT("_r"))) == 0.0;
			bool IsRefLeftPrefix = RefBoneNameStr.Left(2).Compare(FString(TEXT("L_"))) == 0.0;
			bool IsRefRightPrefix = RefBoneNameStr.Left(2).Compare(FString(TEXT("R_"))) == 0.0;
			*/

			UE_LOG(LogTemp, Warning, TEXT("Bone Name: %s"), *RefBoneNameStr);

			/*
			if (IsRefLeftSuffix || IsRefRightSuffix || IsRefLeftPrefix || IsRefRightPrefix) {
				FString RefBoneNameRoot = RefBoneNameStr.LeftChop(2);
				if (IsRefLeftPrefix || IsRefRightPrefix) {
					RefBoneNameRoot = RefBoneNameStr.RightChop(2);
				}
				FString RefBoneNameReversed = RefBoneNameRoot;
				if (IsRefLeftSuffix) {
					RefBoneNameReversed = RefBoneNameRoot + FString(TEXT("_r"));
				}
				else if (IsRefRightSuffix) {
					RefBoneNameReversed = RefBoneNameRoot + FString(TEXT("_l"));
				}
				else if (IsRefLeftPrefix) {
					RefBoneNameReversed = FString(TEXT("R_")) + RefBoneNameRoot;
				}
				else if (IsRefRightPrefix) {
					RefBoneNameReversed = FString(TEXT("L_")) + RefBoneNameRoot;
				}
				UE_LOG(LogTemp, Warning, TEXT("Mirror Name: %s"), *RefBoneNameReversed);
				MirrorExport->SourceBoneName = FName(*RefBoneNameReversed);
			}
			*/

			if (NameTrimOffset != 0) {
				FString RefBoneNameRoot;
				if (NameTrimOffset > 0) {
					RefBoneNameRoot = RefBoneNameStr.RightChop(NameTrimOffset);
				} else {
					RefBoneNameRoot = RefBoneNameStr.LeftChop(-1 * NameTrimOffset);
				}
				FString RefBoneNameReversed = RefBoneNameRoot;
				if (NameTrimOffset > 0) {
					RefBoneNameReversed = ReversedDirection + RefBoneNameRoot;
				} else {
					RefBoneNameReversed = RefBoneNameRoot + ReversedDirection;
				}
				UE_LOG(LogTemp, Warning, TEXT("Mirror Name: %s"), *RefBoneNameReversed);
				MirrorExport->SourceBoneName = FName(*RefBoneNameReversed);
			}
		}

		SkeletalMesh->ImportMirrorTable(MirrorTable);

		SkeletalMeshPackage->SetDirtyFlag(true);
		SkeletalMeshPackage->MarkPackageDirty();

		FText DialogText = FText::FromString(
			TEXT("DONE. Mirror Table populated. Save your mesh!")
		);
		FMessageDialog::Open(EAppMsgType::Ok, DialogText);
	}

	/* 
	FText DialogText = FText::Format(
							LOCTEXT("PluginButtonDialogText", "Selection Count: {0}, Asset Class: {1}"),
							FText::FromString(FString::FromInt(SelectionCount)),
							FText::FromName(AssetClass)
					   );
	FMessageDialog::Open(EAppMsgType::Ok, DialogText);
	*/
}

void FMirrorPluginToolbarModule::AddMenuExtension(FMenuBuilder& Builder)
{
	Builder.AddMenuEntry(FMirrorPluginToolbarCommands::Get().PluginAction);
}

void FMirrorPluginToolbarModule::AddToolbarExtension(FToolBarBuilder& Builder)
{
	Builder.AddToolBarButton(FMirrorPluginToolbarCommands::Get().PluginAction);
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FMirrorPluginToolbarModule, MirrorPluginToolbar)